@@ -18,11 +18,12 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: cgi.ml,v 1.15 2004/09/27 11:56:41 rwmj Exp $
+ * $Id: cgi.ml,v 1.17 2004/10/23 11:49:57 rwmj Exp $
  *)
 
 open Apache
 open Cgi_escape
+open Mod_caml_config
 
 (* Maximum size for POSTed requests. *)
 let post_max = ref max_int
@@ -314,7 +315,7 @@ class cgi r =
     method template ?content_type ?cookie ?cookies
         (template : Template.template) =
       self#header ?content_type ?cookie ?cookies ();
-      print_string r template#to_string
+      ignore (print_string r template#to_string)
 
     method redirect ?cookie ?cookies url =
       self#send_cookies ?cookie ?cookies true;
@@ -478,8 +479,8 @@ struct
 
   exception Sendmail_failure of string
 
-  let sendmail = ref "/usr/sbin/sendmail"
-  let sendmail_args = ref "-t -i"
+  let sendmail = ref mailer_path
+  let sendmail_args = ref mailer_args
   let send () =
     let cmd = !sendmail ^ " " ^ !sendmail_args in
     open_process_out cmd
