@@ -15,7 +15,7 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: mod_caml_c.c,v 1.8 2004/10/23 11:49:57 rwmj Exp $
+ * $Id: mod_caml_c.c,v 1.9 2004/11/19 16:42:25 rwmj Exp $
  */
 
 #include "config.h"
@@ -107,6 +107,43 @@ module_init (server_rec *s, pool *p)
 }
 #endif
 
+/* Notes on per-server and per-directory configuration.
+ *
+ * The actual OCaml data that we store here is defined in the immutable
+ * structs Mod_caml.server_config_t and Mod_caml.dir_config_t.  You are
+ * advised to go and look at those now ...
+ *
+ * Apache lets us store a pointer (void *) and leaves it up to us
+ * what this actually points to.  In our case the pointer (v in the
+ * diagram) points to an OCaml value.  The OCaml value is presumed
+ * to be a pointer to the structure itself.
+ *
+ *              4/8 bytes
+ *              in pool            allocated on OCaml heap
+ *             +----------+       +-- the structure ---------+
+ *    v ---->  | value   -------> | either server_config_t   |
+ *             +----------+       | or dir_config_t          |
+ *                                | ...                      |
+ *                                | ...                      |
+ *                                | ...                      |
+ *                                +--------------------------+
+ *
+ * The value is allocated in the pool which Apache gives us.  This
+ * pool should usually last for the lifetime of the server, but it's
+ * up to Apache to sort that out.
+ *
+ * In addition, we make the value be an OCaml GC global root.  This
+ * means that the GC won't free up the structure prematurely.
+ *
+ * Also we register a cleanup function in the pool.  When Apache does
+ * finally deallocate the pool, our cleanup function will remove the
+ * global root.
+ *
+ * Note that the structure is immutable, so, for example,
+ * cmd_translate_handler and friends actually create a new structure
+ * and update the value to point to the new structure.
+ */
+
 /* Allocate a global root value in the pool, and register a cleanup which
  * will be called when the pool dies to unregister the global root.
  */
@@ -249,6 +286,7 @@ cmd_translate_handler (cmd_parms *parms, void *dummy, const char *name)
   if (Is_exception_result (exn))
     /* XXX ap_pstrdup into pool? */
     return caml_format_exception (Extract_exception (exn));
+  *sconfig = exn;		/* Update server config. */
   return NULL;
 }
 
@@ -262,6 +300,7 @@ name (cmd_parms *parms, void *dconfig, const char *name)	\
   if (Is_exception_result (exn))				\
     /* XXX ap_pstrdup into pool? */				\
     return caml_format_exception (Extract_exception (exn));	\
+  *(value *)dconfig = exn;	/* Update dir config. */        \
   return NULL;							\
 }
 
