@@ -16,7 +16,7 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: template.ml,v 1.2 2004/04/16 15:07:45 rwmj Exp $
+ * $Id: template.ml,v 1.3 2004/10/21 21:32:30 rwmj Exp $
  *)
 
 open Cgi_escape
@@ -94,7 +94,13 @@ let matches str rex =
   with
       Not_found -> None
 
-let compile_template source =
+let compile_template ?filename source =
+  (* Work out the base for file includes. *)
+  let base =
+    match filename with
+      | None -> Filename.current_dir_name
+      | Some filename -> Filename.dirname filename in
+
   (* This function splits up source into a list like this:
    * source = "some text ::tag:: more text ::end::" would become:
    * [ "some text "; "::tag::"; " more text "; "::end::" ]
@@ -136,6 +142,11 @@ let compile_template source =
 	(match is_include x with
 	     None -> [x]
 	   | Some filename ->
+	       let filename =
+		 if Filename.is_relative filename then
+		   Filename.concat base filename
+		 else
+		   filename in
 	       let chan = open_in filename in
 	       let source = read_whole_chan chan in
 	       close_in chan;
@@ -392,7 +403,7 @@ let ct_to_buffer buffer vars compiled =
   let initial_ctx = [vars] in
   loop initial_ctx compiled
 
-class template source =
+class template ?filename source =
   (* This will store the values of set variables/tables. The mutable
    * properties of the hash table make this more useful inside the
    * class. When we print the template we convert this top-level
@@ -402,7 +413,7 @@ class template source =
   let vars = Hashtbl.create 16 in
 
   (* Compile the source. *)
-  let compiled = compile_template source in
+  let compiled = compile_template ?filename source in
 
 object (self)
   method set name value =
@@ -432,15 +443,15 @@ object (self)
 
 end
 
-let template_from_string source =
-  new template source
+let template_from_string ?filename source =
+  new template ?filename source
 
-let template_from_channel chan =
+let template_from_channel ?filename chan =
   let source = read_whole_chan chan in
-  template_from_string source
+  template_from_string ?filename source
 
 let template filename =
   let chan = open_in filename in
-  let template = template_from_channel chan in
+  let template = template_from_channel ~filename chan in
   close_in chan;
   template
